﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Mogre;

namespace TppGame
{
    public class CharacterDecTree : DecTree.FirstSucc
    {
        DecTree.Node PickItemDownSeq;
        DecTree.Job TurnJob;

        public CharacterDecTree()
        {               
            DecTree.FirstSucc resetAnim = new DecTree.FirstSucc(
                new DecTree.Assert(ch => ch.AnimBlender.CurrentAnimSet == "PickItemDown"),
                new DecTree.Job(ch =>
                    {
                        ch.AnimBlender.ResetAnimSet("PickItemDown");
                        ch.AnimBlender.SetAnimSet("PickItemDown");
                        ch.Velocity = Vector3.ZERO;
                        ch.TurnTo(ch.PickingTarget.Position);
                        return true;
                    }));
            Func<Character,bool> animBendDown = (ch => ch.AnimBlender.AnimSetPhase("PickItemDown") > 0.5f);
            DecTree.FirstSucc bendDown = new DecTree.FirstSucc(
                new DecTree.Assert(animBendDown),
                new DecTree.FirstFail(
                    new DecTree.Assert(ch => ch.PickingTarget.Exists),
                    new DecTree.Job(animBendDown)
                    )
                );
            DecTree.FirstSucc pickItem = new DecTree.FirstSucc(
                new DecTree.Assert(ch => ch.PickingTarget == null || !ch.PickingTarget.Exists),
                new DecTree.Job(ch =>
                    {                        
                        ch.Inventory.Add(ch.PickingTarget.Profile);
                        Engine.Singleton.ObjectManager.Destroy(ch.PickingTarget);
                        ch.PickingTarget = null;
                        return true;
                    }));

            DecTree.Job bendUp = new DecTree.Job(ch => ch.AnimBlender.AnimSetPhase("PickItemDown") > 0.99f);

            PickItemDownSeq = new DecTree.FirstFail(
                resetAnim,
                bendDown,
                pickItem,
                bendUp);

            DecTree.Job cleanUp = new DecTree.Job(ch => { ch.PickItemOrder = false; return true; });
            DecTree.FirstFail pickItemNode = new DecTree.FirstFail(
                new DecTree.Assert(ch => ch.PickItemOrder),
                new DecTree.FirstSucc(PickItemDownSeq, cleanUp),
                cleanUp);

            DecTree.FirstFail followPathNode = new DecTree.FirstFail(
                new DecTree.Assert(ch => ch.FollowPathOrder),
                new DecTree.Job(ch =>
                    {                        
                        if (Op2D.Dist(ch.Position, ch.WalkPath[0]) < 0.6f)
                        {
                            ch.WalkPath.RemoveAt(0);
                        }
                        if (ch.WalkPath.Count == 0)
                        {
                            ch.FollowPathOrder = false;
                            return true;
                        }
                        else
                        {
                            ch.Orientation = Quaternion.Slerp(0.2f, ch.Orientation, Vector3.UNIT_Z.GetRotationTo((Op2D.XZ * (ch.WalkPath[0] - ch.Position)).NormalisedCopy), true);
                            ch.Velocity = ch.Orientation * Vector3.UNIT_Z * ch.Profile.WalkSpeed;
                            ch.AnimBlender.SetAnimSet("Walk");
                        }
                        return false;
                    }
            ));


            TurnJob = new DecTree.Job(ch =>
            {
                if (ch.TurnDelta != 0)
                {
                    Quaternion rotation = Quaternion.IDENTITY;
                    rotation.FromAngleAxis(new Degree(ch.TurnDelta), Vector3.UNIT_Y);
                    ch.Orientation *= rotation;
                    ch.TurnDelta = 0;
                }
                return true;
            });
                                            
            DecTree.FirstFail walkNode = new DecTree.FirstFail(
                new DecTree.Assert (ch => ch.MoveOrder),
                new DecTree.Job (ch =>
                {
                    ch.Velocity = ch.Orientation * Vector3.UNIT_Z * ch.Profile.WalkSpeed;
                    ch.AnimBlender.SetAnimSet("Walk");
                    return true;
                }),
                TurnJob);

            DecTree.FirstFail idleNode =  new DecTree.FirstFail(
                new DecTree.Job(ch =>
               {
                    ch.TalkPerm = true;
                    ch.InventoryPerm = true;
                    ch.Velocity = Vector3.ZERO;
                    ch.AnimBlender.SetAnimSet("Idle");
                    return true;
               }),
               TurnJob);

            Children.Add(pickItemNode);
            Children.Add(followPathNode);
            Children.Add(walkNode);
            Children.Add(idleNode);
        }
    }
}
